@@ -26,10 +26,12 @@ extern "C"
 
 #include "rcl/allocator.h"
 #include "rcl/context.h"
+#include "rcl/event_callback.h"
 #include "rcl/guard_condition.h"
 #include "rcl/macros.h"
 #include "rcl/time.h"
 #include "rcl/types.h"
+#include "rcutils/logging_macros.h"
 #include "rmw/rmw.h"
 
 typedef struct rcl_timer_impl_s rcl_timer_impl_t;
@@ -41,6 +43,14 @@ typedef struct rcl_timer_s
   rcl_timer_impl_t * impl;
 } rcl_timer_t;
 
+/// Structure which encapsulates the on reset callback data
+typedef struct rcl_timer_on_reset_callback_data_s
+{
+  rcl_event_callback_t on_reset_callback;
+  const void * user_data;
+  size_t reset_counter;
+} rcl_timer_on_reset_callback_data_t;
+
 /// User callback signature for timers.
 /**
  * The first argument the callback gets is a pointer to the timer.
@@ -116,8 +126,8 @@ rcl_get_zero_initialized_timer(void);
  * // ... error handling
  *
  * rcl_timer_t timer = rcl_get_zero_initialized_timer();
- * ret = rcl_timer_init(
- *   &timer, &clock, context, RCL_MS_TO_NS(100), my_timer_callback, allocator);
+ * ret = rcl_timer_init2(
+ *   &timer, &clock, context, RCL_MS_TO_NS(100), my_timer_callback, allocator, true);
  * // ... error handling, use the timer with a wait set, or poll it manually, then cleanup
  * ret = rcl_timer_fini(&timer);
  * // ... error handling
@@ -142,6 +152,7 @@ rcl_get_zero_initialized_timer(void);
  * \param[in] period the duration between calls to the callback in nanoseconds
  * \param[in] callback the user defined function to be called every period
  * \param[in] allocator the allocator to use for allocations
+ * \param[in] autostart the state of the timer at initialization
  * \return #RCL_RET_OK if the timer was initialized successfully, or
  * \return #RCL_RET_INVALID_ARGUMENT if any arguments are invalid, or
  * \return #RCL_RET_ALREADY_INIT if the timer was already initialized, or
@@ -151,6 +162,22 @@ rcl_get_zero_initialized_timer(void);
 RCL_PUBLIC
 RCL_WARN_UNUSED
 rcl_ret_t
+rcl_timer_init2(
+  rcl_timer_t * timer,
+  rcl_clock_t * clock,
+  rcl_context_t * context,
+  int64_t period,
+  const rcl_timer_callback_t callback,
+  rcl_allocator_t allocator,
+  bool autostart);
+
+/**
+ * \deprecated `rcl_timer_init` implementation was removed.
+ *   Refer to `rcl_timer_init2`.
+ */
+RCL_PUBLIC
+RCUTILS_DEPRECATED_WITH_MSG("Call rcl_timer_init2 instead")
+rcl_ret_t
 rcl_timer_init(
   rcl_timer_t * timer,
   rcl_clock_t * clock,
@@ -225,7 +252,7 @@ rcl_timer_fini(rcl_timer_t * timer);
  * \param[inout] timer the handle to the timer to call
  * \return #RCL_RET_OK if the timer was called successfully, or
  * \return #RCL_RET_INVALID_ARGUMENT if any arguments are invalid, or
- * \return #RCL_RET_TIMER_INVALID if the timer is invalid, or
+ * \return #RCL_RET_TIMER_INVALID if the timer->impl is invalid, or
  * \return #RCL_RET_TIMER_CANCELED if the timer has been canceled, or
  * \return #RCL_RET_ERROR an unspecified error occur.
  */
@@ -250,10 +277,9 @@ rcl_timer_call(rcl_timer_t * timer);
  *
  * \param[in] timer the handle to the timer which is being queried
  * \param[out] clock the rcl_clock_t * in which the clock is stored
- * \return #RCL_RET_OK if the period was retrieved successfully, or
+ * \return #RCL_RET_OK if the clock was retrieved successfully, or
  * \return #RCL_RET_INVALID_ARGUMENT if any arguments are invalid, or
- * \return #RCL_RET_TIMER_INVALID if the timer is invalid, or
- * \return #RCL_RET_ERROR an unspecified error occur.
+ * \return #RCL_RET_TIMER_INVALID if the timer is invalid.
  */
 RCL_PUBLIC
 RCL_WARN_UNUSED
@@ -282,7 +308,7 @@ rcl_timer_clock(rcl_timer_t * timer, rcl_clock_t ** clock);
  * \param[out] is_ready the bool used to store the result of the calculation
  * \return #RCL_RET_OK if the last call time was retrieved successfully, or
  * \return #RCL_RET_INVALID_ARGUMENT if any arguments are invalid, or
- * \return #RCL_RET_TIMER_INVALID if the timer is invalid, or
+ * \return #RCL_RET_TIMER_INVALID if the timer->impl is invalid, or
  * \return #RCL_RET_ERROR an unspecified error occur.
  */
 RCL_PUBLIC
@@ -317,7 +343,8 @@ rcl_timer_is_ready(const rcl_timer_t * timer, bool * is_ready);
  * \param[out] time_until_next_call the output variable for the result
  * \return #RCL_RET_OK if the timer until next call was successfully calculated, or
  * \return #RCL_RET_INVALID_ARGUMENT if any arguments are invalid, or
- * \return #RCL_RET_TIMER_INVALID if the timer is invalid, or
+ * \return #RCL_RET_TIMER_INVALID if the timer->impl is invalid, or
+ * \return #RCL_RET_TIMER_CANCELED if the timer is canceled, or
  * \return #RCL_RET_ERROR an unspecified error occur.
  */
 RCL_PUBLIC
@@ -349,7 +376,7 @@ rcl_timer_get_time_until_next_call(const rcl_timer_t * timer, int64_t * time_unt
  * \param[out] time_since_last_call the struct in which the time is stored
  * \return #RCL_RET_OK if the last call time was retrieved successfully, or
  * \return #RCL_RET_INVALID_ARGUMENT if any arguments are invalid, or
- * \return #RCL_RET_TIMER_INVALID if the timer is invalid, or
+ * \return #RCL_RET_TIMER_INVALID if the timer->impl is invalid, or
  * \return #RCL_RET_ERROR an unspecified error occur.
  */
 RCL_PUBLIC
@@ -376,6 +403,7 @@ rcl_timer_get_time_since_last_call(const rcl_timer_t * timer, int64_t * time_sin
  * \param[out] period the int64_t in which the period is stored
  * \return #RCL_RET_OK if the period was retrieved successfully, or
  * \return #RCL_RET_INVALID_ARGUMENT if any arguments are invalid, or
+ * \return #RCL_RET_TIMER_INVALID if the timer->impl is invalid, or
  * \return #RCL_RET_ERROR an unspecified error occur.
  */
 RCL_PUBLIC
@@ -406,6 +434,7 @@ rcl_timer_get_period(const rcl_timer_t * timer, int64_t * period);
  * \param[out] old_period the int64_t in which the previous period is stored
  * \return #RCL_RET_OK if the period was retrieved successfully, or
  * \return #RCL_RET_INVALID_ARGUMENT if any arguments are invalid, or
+ * \return #RCL_RET_TIMER_INVALID if the timer->impl is invalid, or
  * \return #RCL_RET_ERROR an unspecified error occur.
  */
 RCL_PUBLIC
@@ -481,10 +510,9 @@ rcl_timer_exchange_callback(rcl_timer_t * timer, const rcl_timer_callback_t new_
  * <i>[1] if `atomic_is_lock_free()` returns true for `atomic_int_least64_t`</i>
  *
  * \param[inout] timer the timer to be canceled
- * \return #RCL_RET_OK if the last call time was retrieved successfully, or
+ * \return #RCL_RET_OK if the timer was canceled successfully, or
  * \return #RCL_RET_INVALID_ARGUMENT if any arguments are invalid, or
- * \return #RCL_RET_TIMER_INVALID if the timer is invalid, or
- * \return #RCL_RET_ERROR an unspecified error occur.
+ * \return #RCL_RET_TIMER_INVALID if the timer is invalid.
  */
 RCL_PUBLIC
 RCL_WARN_UNUSED
@@ -512,6 +540,7 @@ rcl_timer_cancel(rcl_timer_t * timer);
  * \param[out] is_canceled storage for the is canceled bool
  * \return #RCL_RET_OK if the last call time was retrieved successfully, or
  * \return #RCL_RET_INVALID_ARGUMENT if any arguments are invalid, or
+ * \return #RCL_RET_TIMER_INVALID if the timer->impl is invalid, or
  * \return #RCL_RET_ERROR an unspecified error occur.
  */
 RCL_PUBLIC
@@ -535,7 +564,7 @@ rcl_timer_is_canceled(const rcl_timer_t * timer, bool * is_canceled);
  * <i>[1] if `atomic_is_lock_free()` returns true for `atomic_int_least64_t`</i>
  *
  * \param[inout] timer the timer to be reset
- * \return #RCL_RET_OK if the last call time was retrieved successfully, or
+ * \return #RCL_RET_OK if the timer was reset successfully, or
  * \return #RCL_RET_INVALID_ARGUMENT if any arguments are invalid, or
  * \return #RCL_RET_TIMER_INVALID if the timer is invalid, or
  * \return #RCL_RET_ERROR an unspecified error occur.
@@ -588,6 +617,34 @@ RCL_WARN_UNUSED
 rcl_guard_condition_t *
 rcl_timer_get_guard_condition(const rcl_timer_t * timer);
 
+/// Set the on reset callback function for the timer.
+/**
+ * This API sets the callback function to be called whenever the
+ * timer is reset.
+ * If the timer has already been reset, the callback will be called.
+ *
+ * <hr>
+ * Attribute          | Adherence
+ * ------------------ | -------------
+ * Allocates Memory   | No
+ * Thread-Safe        | No
+ * Uses Atomics       | No
+ * Lock-Free          | No
+ *
+ * \param[in] timer The handle to the timer on which to set the callback
+ * \param[in] on_reset_callback The callback to be called when timer is reset
+ * \param[in] user_data Given to the callback when called later, may be NULL
+ * \return `RCL_RET_OK` if successful, or
+ * \return `RCL_RET_INVALID_ARGUMENT` if `timer` is NULL
+ */
+RCL_PUBLIC
+RCL_WARN_UNUSED
+rcl_ret_t
+rcl_timer_set_on_reset_callback(
+  const rcl_timer_t * timer,
+  rcl_event_callback_t on_reset_callback,
+  const void * user_data);
+
 #ifdef __cplusplus
 }
 #endif
