@@ -24,6 +24,8 @@
 #include "mimick/mimick.h"
 #include "osrf_testing_tools_cpp/scope_exit.hpp"
 #include "rcl/error_handling.h"
+#include "rcl/node.h"
+#include "rcutils/env.h"
 #include "rmw/validate_full_topic_name.h"
 #include "rmw/validate_node_name.h"
 
@@ -370,6 +372,76 @@ TEST_F(CLASSNAME(TestPublisherFixture, RMW_IMPLEMENTATION), test_publisher_loan)
   }
 }
 
+TEST_F(CLASSNAME(TestPublisherFixture, RMW_IMPLEMENTATION), test_publisher_option) {
+  {
+    rcl_publisher_options_t publisher_options = rcl_publisher_get_default_options();
+    EXPECT_FALSE(publisher_options.disable_loaned_message);
+  }
+  {
+    ASSERT_TRUE(rcutils_set_env("ROS_DISABLE_LOANED_MESSAGES", "0"));
+    rcl_publisher_options_t publisher_options = rcl_publisher_get_default_options();
+    EXPECT_FALSE(publisher_options.disable_loaned_message);
+  }
+  {
+    ASSERT_TRUE(rcutils_set_env("ROS_DISABLE_LOANED_MESSAGES", "1"));
+    rcl_publisher_options_t publisher_options = rcl_publisher_get_default_options();
+    EXPECT_TRUE(publisher_options.disable_loaned_message);
+  }
+  {
+    ASSERT_TRUE(rcutils_set_env("ROS_DISABLE_LOANED_MESSAGES", "2"));
+    rcl_publisher_options_t publisher_options = rcl_publisher_get_default_options();
+    EXPECT_FALSE(publisher_options.disable_loaned_message);
+  }
+  {
+    ASSERT_TRUE(rcutils_set_env("ROS_DISABLE_LOANED_MESSAGES", "Unexpected"));
+    rcl_publisher_options_t publisher_options = rcl_publisher_get_default_options();
+    EXPECT_FALSE(publisher_options.disable_loaned_message);
+  }
+}
+
+TEST_F(CLASSNAME(TestPublisherFixture, RMW_IMPLEMENTATION), test_publisher_loan_disable) {
+  bool is_fastdds = (std::string(rmw_get_implementation_identifier()).find("rmw_fastrtps") == 0);
+  const rosidl_message_type_support_t * ts =
+    ROSIDL_GET_MSG_TYPE_SUPPORT(test_msgs, msg, BasicTypes);
+  constexpr char topic_name[] = "pod_msg";
+
+  {
+    ASSERT_TRUE(rcutils_set_env("ROS_DISABLE_LOANED_MESSAGES", "1"));
+    rcl_publisher_t publisher = rcl_get_zero_initialized_publisher();
+    rcl_publisher_options_t publisher_options = rcl_publisher_get_default_options();
+    EXPECT_TRUE(publisher_options.disable_loaned_message);
+    rcl_ret_t ret =
+      rcl_publisher_init(&publisher, this->node_ptr, ts, topic_name, &publisher_options);
+    ASSERT_EQ(RCL_RET_OK, ret) << rcl_get_error_string().str;
+    OSRF_TESTING_TOOLS_CPP_SCOPE_EXIT(
+    {
+      rcl_ret_t ret = rcl_publisher_fini(&publisher, this->node_ptr);
+      EXPECT_EQ(RCL_RET_OK, ret) << rcl_get_error_string().str;
+    });
+    EXPECT_FALSE(rcl_publisher_can_loan_messages(&publisher));
+  }
+
+  {
+    ASSERT_TRUE(rcutils_set_env("ROS_DISABLE_LOANED_MESSAGES", "0"));
+    rcl_publisher_t publisher = rcl_get_zero_initialized_publisher();
+    rcl_publisher_options_t publisher_options = rcl_publisher_get_default_options();
+    EXPECT_FALSE(publisher_options.disable_loaned_message);
+    rcl_ret_t ret =
+      rcl_publisher_init(&publisher, this->node_ptr, ts, topic_name, &publisher_options);
+    ASSERT_EQ(RCL_RET_OK, ret) << rcl_get_error_string().str;
+    OSRF_TESTING_TOOLS_CPP_SCOPE_EXIT(
+    {
+      rcl_ret_t ret = rcl_publisher_fini(&publisher, this->node_ptr);
+      EXPECT_EQ(RCL_RET_OK, ret) << rcl_get_error_string().str;
+    });
+    if (is_fastdds) {
+      EXPECT_TRUE(rcl_publisher_can_loan_messages(&publisher));
+    } else {
+      EXPECT_FALSE(rcl_publisher_can_loan_messages(&publisher));
+    }
+  }
+}
+
 TEST_F(CLASSNAME(TestPublisherFixture, RMW_IMPLEMENTATION), test_invalid_publisher) {
   rcl_publisher_t publisher = rcl_get_zero_initialized_publisher();
   const rosidl_message_type_support_t * ts =
@@ -419,7 +491,6 @@ TEST_F(CLASSNAME(TestPublisherFixture, RMW_IMPLEMENTATION), test_invalid_publish
   EXPECT_NE(nullptr, rcl_publisher_get_rmw_handle(&publisher));
   EXPECT_NE(nullptr, rcl_publisher_get_actual_qos(&publisher));
   EXPECT_NE(nullptr, rcl_publisher_get_options(&publisher));
-  rcl_reset_error();
   EXPECT_FALSE(rcl_publisher_is_valid(&publisher));
   rcl_reset_error();
   EXPECT_EQ(nullptr, rcl_publisher_get_context(&publisher));
@@ -778,18 +849,22 @@ TEST_F(CLASSNAME(TestPublisherFixture, RMW_IMPLEMENTATION), test_mock_loaned_fun
     EXPECT_EQ(
       RCL_RET_PUBLISHER_INVALID,
       rcl_publish_loaned_message(nullptr, &msg, null_allocation_is_valid_arg));
+    rcl_reset_error();
     EXPECT_EQ(
       RCL_RET_PUBLISHER_INVALID,
       rcl_publish_loaned_message(&not_init_publisher, &msg, null_allocation_is_valid_arg));
+    rcl_reset_error();
     EXPECT_EQ(
       RCL_RET_INVALID_ARGUMENT,
       rcl_publish_loaned_message(&publisher, nullptr, null_allocation_is_valid_arg));
+    rcl_reset_error();
   }
   {
     // mocked, failure publish
     auto mock = mocking_utils::patch_and_return(
       "lib:rcl", rmw_publish_loaned_message, RMW_RET_ERROR);
     EXPECT_EQ(RCL_RET_ERROR, rcl_publish_loaned_message(&publisher, &msg, nullptr));
+    rcl_reset_error();
   }
   {
     // mocked, borrow loaned nominal usage
@@ -799,8 +874,10 @@ TEST_F(CLASSNAME(TestPublisherFixture, RMW_IMPLEMENTATION), test_mock_loaned_fun
   {
     // bad params borrow loaned
     EXPECT_EQ(RCL_RET_PUBLISHER_INVALID, rcl_borrow_loaned_message(nullptr, ts, &msg_pointer));
+    rcl_reset_error();
     EXPECT_EQ(
       RCL_RET_PUBLISHER_INVALID, rcl_borrow_loaned_message(&not_init_publisher, ts, &msg_pointer));
+    rcl_reset_error();
   }
   {
     // mocked, nominal return loaned message
@@ -813,12 +890,15 @@ TEST_F(CLASSNAME(TestPublisherFixture, RMW_IMPLEMENTATION), test_mock_loaned_fun
     EXPECT_EQ(
       RCL_RET_PUBLISHER_INVALID,
       rcl_return_loaned_message_from_publisher(nullptr, &msg));
+    rcl_reset_error();
     EXPECT_EQ(
       RCL_RET_PUBLISHER_INVALID,
       rcl_return_loaned_message_from_publisher(&not_init_publisher, &msg));
+    rcl_reset_error();
     EXPECT_EQ(
       RCL_RET_INVALID_ARGUMENT,
       rcl_return_loaned_message_from_publisher(&publisher, nullptr));
+    rcl_reset_error();
   }
 
   test_msgs__msg__BasicTypes__fini(&msg);
@@ -893,4 +973,5 @@ TEST_F(CLASSNAME(TestPublisherFixture, RMW_IMPLEMENTATION), test_mock_publisher_
   auto mock = mocking_utils::patch_and_return("lib:rcl", rmw_destroy_publisher, RMW_RET_ERROR);
   ret = rcl_publisher_fini(&publisher, this->node_ptr);
   EXPECT_EQ(RCL_RET_ERROR, ret) << rcl_get_error_string().str;
+  rcl_reset_error();
 }
