@@ -92,6 +92,21 @@ TEST_F(CLASSNAME(TestServiceFixture, RMW_IMPLEMENTATION), test_service_nominal)
 
   ret = rcl_service_init(&service, this->node_ptr, ts, topic, &service_options);
   EXPECT_EQ(RCL_RET_ALREADY_INIT, ret) << rcl_get_error_string().str;
+  rcl_reset_error();
+
+  const rmw_qos_profile_t * request_subscription_qos =
+    rcl_service_request_subscription_get_actual_qos(&service);
+  EXPECT_EQ(rmw_qos_profile_services_default.reliability, request_subscription_qos->reliability);
+  EXPECT_EQ(rmw_qos_profile_services_default.history, request_subscription_qos->history);
+  EXPECT_EQ(rmw_qos_profile_services_default.depth, request_subscription_qos->depth);
+  EXPECT_EQ(rmw_qos_profile_services_default.durability, request_subscription_qos->durability);
+
+  const rmw_qos_profile_t * response_publisher_qos =
+    rcl_service_response_publisher_get_actual_qos(&service);
+  EXPECT_EQ(rmw_qos_profile_services_default.reliability, response_publisher_qos->reliability);
+  EXPECT_EQ(rmw_qos_profile_services_default.history, response_publisher_qos->history);
+  EXPECT_EQ(rmw_qos_profile_services_default.depth, response_publisher_qos->depth);
+  EXPECT_EQ(rmw_qos_profile_services_default.durability, response_publisher_qos->durability);
 
   ret = rcl_service_fini(&service, this->node_ptr);
   EXPECT_EQ(RCL_RET_OK, ret) << rcl_get_error_string().str;
@@ -110,7 +125,6 @@ TEST_F(CLASSNAME(TestServiceFixture, RMW_IMPLEMENTATION), test_service_nominal)
   ret = rcl_service_init(&service, this->node_ptr, ts, topic, &service_options);
   EXPECT_EQ(RCL_RET_OK, ret) << rcl_get_error_string().str;
   EXPECT_TRUE(rcl_service_is_valid(&service));
-  rcl_reset_error();
 
   // Check that the service name matches what we assigned.
   EXPECT_EQ(strcmp(rcl_service_get_service_name(&service), expected_topic), 0);
@@ -238,7 +252,6 @@ TEST_F(CLASSNAME(TestServiceFixture, RMW_IMPLEMENTATION), test_service_without_i
   ret = rcl_service_init(&service, this->node_ptr, ts, topic, &service_options);
   EXPECT_EQ(RCL_RET_OK, ret) << rcl_get_error_string().str;
   EXPECT_TRUE(rcl_service_is_valid(&service));
-  rcl_reset_error();
 
   // Check that the service name matches what we assigned.
   EXPECT_STREQ(rcl_service_get_service_name(&service), expected_topic);
@@ -340,33 +353,43 @@ TEST_F(CLASSNAME(TestServiceFixture, RMW_IMPLEMENTATION), test_bad_arguments) {
   EXPECT_EQ(
     RCL_RET_NODE_INVALID, rcl_service_init(
       &service, nullptr, ts, topic, &service_options)) << rcl_get_error_string().str;
+  rcl_reset_error();
   EXPECT_EQ(
     RCL_RET_NODE_INVALID, rcl_service_init(
       &service, &invalid_node, ts, topic, &service_options)) << rcl_get_error_string().str;
+  rcl_reset_error();
   EXPECT_EQ(
     RCL_RET_INVALID_ARGUMENT, rcl_service_init(
       nullptr, this->node_ptr, ts, topic, &service_options)) << rcl_get_error_string().str;
+  rcl_reset_error();
   EXPECT_EQ(
     RCL_RET_INVALID_ARGUMENT, rcl_service_init(
       &service, this->node_ptr, nullptr, topic, &service_options)) << rcl_get_error_string().str;
+  rcl_reset_error();
   EXPECT_EQ(
     RCL_RET_INVALID_ARGUMENT, rcl_service_init(
       &service, this->node_ptr, ts, nullptr, &service_options)) << rcl_get_error_string().str;
+  rcl_reset_error();
   EXPECT_EQ(
     RCL_RET_INVALID_ARGUMENT, rcl_service_init(
       &service, this->node_ptr, ts, topic, nullptr)) << rcl_get_error_string().str;
+  rcl_reset_error();
   EXPECT_EQ(
     RCL_RET_INVALID_ARGUMENT, rcl_service_init(
       &service, this->node_ptr, ts, topic,
       &service_options_bad_alloc)) << rcl_get_error_string().str;
+  rcl_reset_error();
 
   EXPECT_EQ(
     RCL_RET_NODE_INVALID, rcl_service_fini(&service, nullptr)) << rcl_get_error_string().str;
+  rcl_reset_error();
   EXPECT_EQ(
     RCL_RET_NODE_INVALID, rcl_service_fini(&service, &invalid_node)) << rcl_get_error_string().str;
+  rcl_reset_error();
   EXPECT_EQ(
     RCL_RET_SERVICE_INVALID, rcl_service_fini(
       nullptr, this->node_ptr)) << rcl_get_error_string().str;
+  rcl_reset_error();
 
   test_msgs__srv__BasicTypes_Request service_request;
   test_msgs__srv__BasicTypes_Response service_response;
@@ -380,30 +403,52 @@ TEST_F(CLASSNAME(TestServiceFixture, RMW_IMPLEMENTATION), test_bad_arguments) {
   });
 
   EXPECT_EQ(nullptr, rcl_service_get_service_name(nullptr));
+  rcl_reset_error();
   EXPECT_EQ(nullptr, rcl_service_get_options(nullptr));
+  rcl_reset_error();
   EXPECT_EQ(nullptr, rcl_service_get_rmw_handle(nullptr));
+  rcl_reset_error();
   EXPECT_EQ(
     RCL_RET_SERVICE_INVALID, rcl_take_request_with_info(nullptr, &header, &service_request));
+  rcl_reset_error();
   EXPECT_EQ(
     RCL_RET_SERVICE_INVALID, rcl_send_response(nullptr, &header.request_id, &service_response));
+  rcl_reset_error();
   EXPECT_EQ(
     RCL_RET_SERVICE_INVALID, rcl_take_request(nullptr, &(header.request_id), &service_request));
+  rcl_reset_error();
+  EXPECT_EQ(nullptr, rcl_service_request_subscription_get_actual_qos(nullptr));
+  rcl_reset_error();
+  EXPECT_EQ(nullptr, rcl_service_response_publisher_get_actual_qos(nullptr));
+  rcl_reset_error();
 
   EXPECT_EQ(nullptr, rcl_service_get_service_name(&service));
+  rcl_reset_error();
   EXPECT_EQ(nullptr, rcl_service_get_options(&service));
+  rcl_reset_error();
   EXPECT_EQ(nullptr, rcl_service_get_rmw_handle(&service));
+  rcl_reset_error();
   EXPECT_EQ(
     RCL_RET_SERVICE_INVALID, rcl_take_request_with_info(&service, &header, &service_request));
+  rcl_reset_error();
   EXPECT_EQ(
     RCL_RET_SERVICE_INVALID, rcl_send_response(&service, &(header.request_id), &service_response));
+  rcl_reset_error();
   EXPECT_EQ(
     RCL_RET_SERVICE_INVALID, rcl_take_request(&service, &(header.request_id), &service_request));
+  rcl_reset_error();
 
   service_options_bad_alloc.allocator = get_failing_allocator();
   EXPECT_EQ(
     RCL_RET_BAD_ALLOC, rcl_service_init(
       &service, this->node_ptr, ts,
       topic, &service_options_bad_alloc)) << rcl_get_error_string().str;
+  rcl_reset_error();
+
+  EXPECT_EQ(nullptr, rcl_service_request_subscription_get_actual_qos(&service));
+  rcl_reset_error();
+  EXPECT_EQ(nullptr, rcl_service_response_publisher_get_actual_qos(&service));
+  rcl_reset_error();
 }
 
 /* Name failed tests
@@ -614,12 +659,15 @@ TEST_F(CLASSNAME(TestServiceFixture, RMW_IMPLEMENTATION), test_fail_send_respons
 
   ret = rcl_send_response(nullptr, &header.request_id, &service_response);
   EXPECT_EQ(RCL_RET_SERVICE_INVALID, ret);
+  rcl_reset_error();
 
   ret = rcl_send_response(&service, nullptr, &service_response);
   EXPECT_EQ(RCL_RET_INVALID_ARGUMENT, ret);
+  rcl_reset_error();
 
   ret = rcl_send_response(&service, &header.request_id, nullptr);
   EXPECT_EQ(RCL_RET_INVALID_ARGUMENT, ret);
+  rcl_reset_error();
 
   {
     auto mock = mocking_utils::patch_and_return(
