@@ -26,19 +26,12 @@
 
 #include "./allocator_testing_utils.h"
 
-#ifdef RMW_IMPLEMENTATION
-# define CLASSNAME_(NAME, SUFFIX) NAME ## __ ## SUFFIX
-# define CLASSNAME(NAME, SUFFIX) CLASSNAME_(NAME, SUFFIX)
-#else
-# define CLASSNAME(NAME, SUFFIX) NAME
-#endif
-
 using osrf_testing_tools_cpp::memory_tools::on_unexpected_malloc;
 using osrf_testing_tools_cpp::memory_tools::on_unexpected_realloc;
 using osrf_testing_tools_cpp::memory_tools::on_unexpected_calloc;
 using osrf_testing_tools_cpp::memory_tools::on_unexpected_free;
 
-class CLASSNAME (TestTimeFixture, RMW_IMPLEMENTATION) : public ::testing::Test
+class TestTimeFixture : public ::testing::Test
 {
 public:
   void SetUp()
@@ -57,7 +50,7 @@ public:
 };
 
 // Tests the rcl_set_ros_time_override() function.
-TEST_F(CLASSNAME(TestTimeFixture, RMW_IMPLEMENTATION), test_rcl_ros_time_set_override) {
+TEST_F(TestTimeFixture, test_rcl_ros_time_set_override) {
   osrf_testing_tools_cpp::memory_tools::enable_monitoring_in_all_threads();
   rcl_clock_t ros_clock;
   rcl_allocator_t allocator = rcl_get_default_allocator();
@@ -161,7 +154,7 @@ TEST_F(CLASSNAME(TestTimeFixture, RMW_IMPLEMENTATION), test_rcl_ros_time_set_ove
   }
 }
 
-TEST_F(CLASSNAME(TestTimeFixture, RMW_IMPLEMENTATION), test_rcl_init_for_clock_and_point) {
+TEST_F(TestTimeFixture, test_rcl_init_for_clock_and_point) {
   rcl_ret_t ret;
   rcl_allocator_t allocator = rcl_get_default_allocator();
   // Check for invalid argument error condition (allowed to alloc).
@@ -191,7 +184,7 @@ TEST_F(CLASSNAME(TestTimeFixture, RMW_IMPLEMENTATION), test_rcl_init_for_clock_a
   });
 }
 
-TEST_F(CLASSNAME(TestTimeFixture, RMW_IMPLEMENTATION), test_ros_clock_initially_zero) {
+TEST_F(TestTimeFixture, test_ros_clock_initially_zero) {
   rcl_allocator_t allocator = rcl_get_default_allocator();
   rcl_clock_t ros_clock;
   ASSERT_EQ(RCL_RET_OK, rcl_ros_clock_init(&ros_clock, &allocator)) << rcl_get_error_string().str;
@@ -205,7 +198,7 @@ TEST_F(CLASSNAME(TestTimeFixture, RMW_IMPLEMENTATION), test_ros_clock_initially_
   EXPECT_EQ(0, query_now);
 }
 
-TEST(CLASSNAME(rcl_time, RMW_IMPLEMENTATION), clock_validation) {
+TEST(rcl_time, clock_validation) {
   ASSERT_FALSE(rcl_clock_valid(NULL));
   rcl_clock_t uninitialized;
   // Not reliably detectable due to random values.
@@ -220,7 +213,7 @@ TEST(CLASSNAME(rcl_time, RMW_IMPLEMENTATION), clock_validation) {
   });
 }
 
-TEST(CLASSNAME(rcl_time, RMW_IMPLEMENTATION), default_clock_instanciation) {
+TEST(rcl_time, default_clock_instanciation) {
   rcl_clock_t ros_clock;
   rcl_allocator_t allocator = rcl_get_default_allocator();
   rcl_ret_t retval = rcl_ros_clock_init(&ros_clock, &allocator);
@@ -261,7 +254,7 @@ TEST(CLASSNAME(rcl_time, RMW_IMPLEMENTATION), default_clock_instanciation) {
   ASSERT_TRUE(rcl_clock_valid(system_clock));
 }
 
-TEST(CLASSNAME(rcl_time, RMW_IMPLEMENTATION), specific_clock_instantiation) {
+TEST(rcl_time, specific_clock_instantiation) {
   rcl_allocator_t allocator = rcl_get_default_allocator();
   {
     rcl_clock_t uninitialized_clock;
@@ -318,7 +311,42 @@ TEST(CLASSNAME(rcl_time, RMW_IMPLEMENTATION), specific_clock_instantiation) {
   }
 }
 
-TEST(CLASSNAME(rcl_time, RMW_IMPLEMENTATION), rcl_time_difference) {
+TEST(rcl_time, rcl_clock_time_started) {
+  rcl_allocator_t allocator = rcl_get_default_allocator();
+  {
+    rcl_clock_t ros_clock;
+    rcl_ret_t ret = rcl_clock_init(RCL_ROS_TIME, &ros_clock, &allocator);
+    ASSERT_EQ(ret, RCL_RET_OK) << rcl_get_error_string().str;
+    // At this point, the ROS clock is reading system time since the ROS time override isn't on
+    // So we expect it to be started (it's extremely unlikely that system time is at epoch start)
+    ASSERT_TRUE(rcl_clock_time_started(&ros_clock));
+    ASSERT_EQ(RCL_RET_OK, rcl_enable_ros_time_override(&ros_clock)) << rcl_get_error_string().str;
+    ASSERT_FALSE(rcl_clock_time_started(&ros_clock));
+    ret = rcl_set_ros_time_override(&ros_clock, 1);
+    EXPECT_EQ(ret, RCL_RET_OK) << rcl_get_error_string().str;
+    ASSERT_TRUE(rcl_clock_time_started(&ros_clock));
+    ret = rcl_clock_fini(&ros_clock);
+    EXPECT_EQ(ret, RCL_RET_OK) << rcl_get_error_string().str;
+  }
+  {
+    rcl_clock_t system_clock;
+    rcl_ret_t ret = rcl_clock_init(RCL_SYSTEM_TIME, &system_clock, &allocator);
+    ASSERT_EQ(ret, RCL_RET_OK) << rcl_get_error_string().str;
+    ASSERT_TRUE(rcl_clock_time_started(&system_clock));  // As long as system time isn't 0
+    ret = rcl_clock_fini(&system_clock);
+    EXPECT_EQ(ret, RCL_RET_OK) << rcl_get_error_string().str;
+  }
+  {
+    rcl_clock_t steady_clock;
+    rcl_ret_t ret = rcl_clock_init(RCL_STEADY_TIME, &steady_clock, &allocator);
+    ASSERT_EQ(ret, RCL_RET_OK) << rcl_get_error_string().str;
+    ASSERT_TRUE(rcl_clock_time_started(&steady_clock));
+    ret = rcl_clock_fini(&steady_clock);
+    EXPECT_EQ(ret, RCL_RET_OK) << rcl_get_error_string().str;
+  }
+}
+
+TEST(rcl_time, rcl_time_difference) {
   rcl_ret_t ret;
   rcl_time_point_t a, b;
 
@@ -342,7 +370,7 @@ TEST(CLASSNAME(rcl_time, RMW_IMPLEMENTATION), rcl_time_difference) {
   rcl_reset_error();
 }
 
-TEST(CLASSNAME(rcl_time, RMW_IMPLEMENTATION), rcl_time_difference_signed) {
+TEST(rcl_time, rcl_time_difference_signed) {
   rcl_time_point_t a, b;
   a.nanoseconds = RCL_S_TO_NS(0LL) + 0LL;
   b.nanoseconds = RCL_S_TO_NS(10LL) + 0LL;
@@ -411,7 +439,7 @@ void reset_callback_triggers(void)
   post_callback_called = false;
 }
 
-TEST(CLASSNAME(rcl_time, RMW_IMPLEMENTATION), rcl_time_clock_change_callbacks) {
+TEST(rcl_time, rcl_time_clock_change_callbacks) {
   rcl_allocator_t allocator = rcl_get_default_allocator();
   rcl_clock_t ros_clock;
   rcl_ret_t ret = rcl_ros_clock_init(&ros_clock, &allocator);
@@ -471,7 +499,7 @@ TEST(CLASSNAME(rcl_time, RMW_IMPLEMENTATION), rcl_time_clock_change_callbacks) {
   reset_callback_triggers();
 }
 
-TEST(CLASSNAME(rcl_time, RMW_IMPLEMENTATION), rcl_time_fail_set_jump_callbacks) {
+TEST(rcl_time, rcl_time_fail_set_jump_callbacks) {
   rcl_allocator_t allocator = rcl_get_default_allocator();
   rcl_clock_t fail_clock;
   rcl_time_jump_t time_jump;
@@ -498,7 +526,7 @@ TEST(CLASSNAME(rcl_time, RMW_IMPLEMENTATION), rcl_time_fail_set_jump_callbacks)
   rcl_reset_error();
 }
 
-TEST(CLASSNAME(rcl_time, RMW_IMPLEMENTATION), rcl_time_forward_jump_callbacks) {
+TEST(rcl_time, rcl_time_forward_jump_callbacks) {
   rcl_clock_t ros_clock;
   rcl_allocator_t allocator = rcl_get_default_allocator();
   rcl_ret_t ret = rcl_ros_clock_init(&ros_clock, &allocator);
@@ -562,7 +590,7 @@ TEST(CLASSNAME(rcl_time, RMW_IMPLEMENTATION), rcl_time_forward_jump_callbacks) {
   EXPECT_FALSE(post_callback_called);
 }
 
-TEST(CLASSNAME(rcl_time, RMW_IMPLEMENTATION), rcl_time_backward_jump_callbacks) {
+TEST(rcl_time, rcl_time_backward_jump_callbacks) {
   rcl_clock_t ros_clock;
   rcl_allocator_t allocator = rcl_get_default_allocator();
   rcl_ret_t ret = rcl_ros_clock_init(&ros_clock, &allocator);
@@ -625,7 +653,7 @@ TEST(CLASSNAME(rcl_time, RMW_IMPLEMENTATION), rcl_time_backward_jump_callbacks)
   EXPECT_FALSE(post_callback_called);
 }
 
-TEST(CLASSNAME(rcl_time, RMW_IMPLEMENTATION), rcl_clock_add_jump_callback) {
+TEST(rcl_time, rcl_clock_add_jump_callback) {
   rcl_clock_t clock;
   rcl_allocator_t allocator = rcl_get_default_allocator();
   rcl_ret_t ret = rcl_ros_clock_init(&clock, &allocator);
@@ -659,7 +687,7 @@ TEST(CLASSNAME(rcl_time, RMW_IMPLEMENTATION), rcl_clock_add_jump_callback) {
   EXPECT_EQ(2u, clock.num_jump_callbacks);
 }
 
-TEST(CLASSNAME(rcl_time, RMW_IMPLEMENTATION), rcl_clock_remove_jump_callback) {
+TEST(rcl_time, rcl_clock_remove_jump_callback) {
   rcl_clock_t clock;
   rcl_allocator_t allocator = rcl_get_default_allocator();
   rcl_ret_t ret = rcl_ros_clock_init(&clock, &allocator);
@@ -704,7 +732,7 @@ TEST(CLASSNAME(rcl_time, RMW_IMPLEMENTATION), rcl_clock_remove_jump_callback) {
   EXPECT_EQ(0u, clock.num_jump_callbacks);
 }
 
-TEST(CLASSNAME(rcl_time, RMW_IMPLEMENTATION), add_remove_add_jump_callback) {
+TEST(rcl_time, add_remove_add_jump_callback) {
   rcl_allocator_t failing_allocator = get_failing_allocator();
   set_failing_allocator_is_failing(failing_allocator, false);
 
@@ -755,7 +783,7 @@ TEST(CLASSNAME(rcl_time, RMW_IMPLEMENTATION), add_remove_add_jump_callback) {
   EXPECT_EQ(1u, clock.num_jump_callbacks);
 }
 
-TEST(CLASSNAME(rcl_time, RMW_IMPLEMENTATION), failed_get_now) {
+TEST(rcl_time, failed_get_now) {
   rcl_allocator_t allocator = rcl_get_default_allocator();
   rcl_clock_t uninitialized_clock;
   rcl_time_point_value_t query_now;
@@ -767,7 +795,7 @@ TEST(CLASSNAME(rcl_time, RMW_IMPLEMENTATION), failed_get_now) {
   rcl_reset_error();
 }
 
-TEST(CLASSNAME(rcl_time, RMW_IMPLEMENTATION), fail_ros_time_override) {
+TEST(rcl_time, fail_ros_time_override) {
   bool result;
   rcl_time_point_value_t set_point = 1000000000ull;
 
